#pragma once


#include <iomanip>
#include "MeshLoader.h"

template<uint8_t DIM>
std::ofstream& operator<<(std::ofstream& p_out, const Node<DIM>& node)
{
	p_out << "\nID: " << node.id;
	p_out << "\nCoordinates: [ ";
	for (uint8_t i = 0; i < DIM - 1; ++i) p_out << std::showpoint << node.coord[i] << ", ";
	p_out << node.coord[DIM - 1] << " ]";
	p_out << "\nBoundary node: ";
    // if (node.in_node)
    // СРАВНИВАТЬ С КОНСТАНТОЙ ИЗБЫТОЧНО, УЖЕ ИМЕЕМ ПЕРЕМЕННУЮ ТИПА bool
	if (node.in_node) p_out << "NO";
	else p_out << "YES";

	return p_out;
}

/* it doesn't work
template<uint8_t DIM>
void printNode(const MeshLoader<DIM>& ml, const uint32_t& node_id)
{
	//printNode(ml->getNodeWithID(node_id));
}
*/

template <uint8_t DIM, typename IterType>
void printNodes(const IterType& it1, const IterType& it2)
{
	std::cout << "\n+------------+--";
	for (uint8_t i = 0; i < DIM; ++i) std::cout << "-----------";
	std::cout << "+-------------+";
	std::cout << "\n|     ID     |  ";
	std::cout << "COORDINATES";
	for (uint8_t i = 0; i < DIM - 1; ++i) std::cout << "           ";
	std::cout << "| IS BOUNDARY | ";
	std::cout << "\n+------------+--";
	for (uint8_t i = 0; i < DIM; ++i) std::cout << "-----------";
	std::cout << "+-------------+";

	for (IterType it = it1; it != it2; ++it)
	{
		std::cout << "\n| " << std::setw(10) << it->id << " |  ";
		for (uint8_t i = 0; i < DIM - 1; ++i) std::cout << std::setw(9) << std::setprecision(5) << std::showpoint << it->coord[i] << ", ";
		std::cout << std::setw(9) << std::setprecision(5) << std::showpoint << it->coord[DIM - 1] << "  | ";
		if (it->in_node == true) std::cout << "    NO      |";
		else std::cout << "    YES     |";
		std::cout << "\n+------------+--";
		for (uint8_t i = 0; i < DIM; ++i) std::cout << "-----------";
		std::cout << "+-------------+";
	}
}


std::ofstream& operator<<(std::ofstream& p_out, const FiniteElement& fe)
{
	p_out << "\nID: " << fe.id;
	p_out << "\nMaterial ID: " << fe.material_id;
	p_out << "\nNodes: ";
	for (uint8_t i = 0; i < fe.nodes.size() - 1; ++i) p_out << fe.nodes[i] << ", ";
	p_out << fe.nodes[fe.nodes.size() - 1];

	return p_out;
}


template <typename IterType>
void printFEs(const IterType& it1, const IterType& it2)
{
	uint8_t fe_vertex_cnt = it1->nodes.size(); //we take size of finite element vector "nodes"(we take it from element "it1" just for simplicity)


	std::cout << "\n+------------+-------------+--";
	for (uint8_t i = 0; i < fe_vertex_cnt; ++i) std::cout << "------------";
	std::cout << "+";
	std::cout << "\n|     ID     | Material ID |  NODES     ";
	for (uint8_t i = 0; i < fe_vertex_cnt - 1; ++i) std::cout << "            ";
	std::cout << "  |";
	std::cout << "\n+------------+-------------+--";
	for (uint8_t i = 0; i < fe_vertex_cnt; ++i) std::cout << "------------";
	std::cout << "+";

	for (IterType it = it1; it != it2; ++it)
	{
		std::cout << "\n| " << std::setw(10) << it->id;
		std::cout << " | " << std::setw(11) << it->material_id << " |  ";
		for (uint8_t i = 0; i < fe_vertex_cnt - 1; ++i) std::cout << std::setw(10) << it->nodes[i] << ", ";
		std::cout << std::setw(10) << it->nodes[fe_vertex_cnt - 1] << "  | ";
		std::cout << "\n+------------+-------------+--";
		for (uint8_t i = 0; i < fe_vertex_cnt; ++i) std::cout << "------------";
		std::cout << "+";
	}
}


std::ofstream& operator<<(std::ofstream& p_out, const BoundaryFiniteElement& bfe)
{
	std::cout << "\nID: " << bfe.id;
	std::cout << "\nBoundary ID: " << bfe.boundary_id;
	std::cout << "\nNodes: ";
	for (uint8_t i = 0; i < bfe.nodes.size() - 1; ++i) std::cout << bfe.nodes[i] << ", ";
	std::cout << bfe.nodes[bfe.nodes.size() - 1];

	return p_out;
}


template <typename IterType>
void printBFEs(const IterType& it1, const IterType& it2)
{

	uint8_t bfe_vertex_cnt = it1->nodes.size(); //we take size of boundary finite element vector "nodes"(we take it from element "it1" just for simplicity)


	std::cout << "\n+------------+-------------+--";
	for (uint8_t i = 0; i < bfe_vertex_cnt; ++i) std::cout << "------------";
	std::cout << "+";
	std::cout << "\n|     ID     | Boundary ID |  NODES       ";
	for (uint8_t i = 0; i < bfe_vertex_cnt - 1; ++i) std::cout << "            ";
	std::cout << "|";
	std::cout << "\n+------------+-------------+--";
	for (uint8_t i = 0; i < bfe_vertex_cnt; ++i) std::cout << "------------";
	std::cout << "+";

	for (IterType it = it1; it != it2; ++it)
	{
		std::cout << "\n| " << std::setw(10) << it->id;
		std::cout << " | " << std::setw(11) << it->boundary_id << " |  ";
		for (uint8_t i = 0; i < bfe_vertex_cnt - 1; ++i) std::cout << std::setw(10) << it->nodes[i] << ", ";
		std::cout << std::setw(10) << it->nodes[bfe_vertex_cnt - 1] << "  | ";
		std::cout << "\n+------------+-------------+--";
		for (uint8_t i = 0; i < bfe_vertex_cnt; ++i) std::cout << "------------";
		std::cout << "+";
	}
}


void printMenu()
{
	std::cout << "1 - find finite elements that contain nodes with given ID;\n";
	std::cout << "2 - find finite elements that contain an edge with given ID of two nodes;\n";
	std::cout << "3 - find boundary nodes with given boundary ID;\n";
	std::cout << "4 - find finite elements with given material ID;\n";
	std::cout << "5 - find boundary finite elements with given boundary ID;\n";
	std::cout << "6 - find neighbours of concrete node;\n";
	std::cout << "7 - insert new nodes in edges center of all finite elements;\n";
	std::cout << "8 - print all nodes;\n";
	std::cout << "9 - print nodes in concrete diapazon;\n";
	std::cout << "10 - print all finite elements;\n";
	std::cout << "11 - print finite elements in concrete diapazon;\n";
	std::cout << "12 - print all boundary finite elements;\n";
	std::cout << "13 - print boundary finite elements with given boundary ID(without recording into container);\n";
	std::cout << "14 - quit.\n";
}
