#include <iostream>
#include <string>
#include "AneuMeshLoader.h"
#include "Exception.h"
#include "Print_Functions.h"


int main(int argc, char* argv[])
{
    // ЗАДАНИЕ:
    // ВЫВЕСТИ НА ЭКРАН ПРИ ПОМОЩИ АЛГОРИТМОВ И ПОТОКОВЫХ ИТЕРАТОРОВ
    // ГРАНИЧНЫЕ ПОВЕРХНОСТИ С ЗАДАННЫМ НОМЕРОМ
	//********Work with file********

	std::string filename; //file with data
	AneuMeshLoader<3> aml;


	if (argc <= 1)
	{
		std::cout << "File name was not detected. Please enter file name: ";
		std::cin >> filename;
	}
	else
	{
		filename = argv[1];
	}

	std::cout << "Opening file " << filename << "...\n";
	
	try
	{
		aml.loadMesh(filename);
		
		//********Work with user********

		int ch = 0; //number of menu item(0 - default value)

		while (ch != 14)
		{
			std::cout << "\n\nChoose what do you want to do(input one of the following numbers):\n";
			printMenu();

			std::cout << "\n-> ";
			std::cin >> ch;
			
			try
			{
				switch (ch)
				{
				case 1:
				{
					uint32_t id1, id2, id3; //ID of nodes
					std::cout << "\nInput ID:\n";
					std::cout << "ID1: "; std::cin >> id1;
					std::cout << "ID2: "; std::cin >> id2;
					std::cout << "ID3: "; std::cin >> id3;

					std::cout << "\n\nFinite elements with nodes " << id1 << " " << id2 << " " << id3 << ":\n";
					std::vector<FiniteElement> buffer = aml.findFE(id1, id2, id3);

					break;
				}
				
				case 2:
				{
					uint32_t id1, id2; //ID of nodes
					std::cout << "\nInput ID:\n";
					std::cout << "ID1: "; std::cin >> id1;
					std::cout << "ID2: "; std::cin >> id2;

					std::cout << "\n\nFinite elements with nodes " << id1 << " " << id2 << ":\n";
					std::vector<FiniteElement> buffer = aml.findFE(id1, id2);

					break;
				}
				
				case 3:
				{
					uint32_t b_id; //boundary ID
					std::cout << "\nInput boundary ID: "; std::cin >> b_id;
					std::cout << "\n\nBoundary nodes with boundary id " << b_id << ":\n";
					std::vector<Node<3>> buffer = aml.getBoundaryNodes(b_id);

					break;
				}
				
				case 4:
				{
					uint16_t mater_id; //material ID
					std::cout << "\nInput material ID: "; std::cin >> mater_id;
					std::cout << "\n\nFinite elements with material id " << mater_id << ":\n";
					std::vector<FiniteElement> buffer = aml.getMaterialFEs(mater_id);

					break;
				}
				
				case 5:
				{
					uint32_t b_id; //boundary ID
					std::cout << "\nInput boundary ID: "; std::cin >> b_id;
					std::cout << "\n\nBoundary finite elements with boundary id " << b_id << ":\n";
					std::vector<BoundaryFiniteElement> buffer = aml.getBoundaryBFEs(b_id);

					break;
				}
				
				case 6:
				{
					uint32_t node_id; //node ID
					std::cout << "Nodes ID are in diapazon [1, " << aml.getNodeCnt() << "]\n";
					std::cout << "\nInput node ID: "; std::cin >> node_id;
					if ((node_id <= 0) || (node_id > aml.getNodeCnt()))
					{
						throw NodeWrongIDExc();
					}
					else
					{
						std::cout << "\n\nNeighbours of node " << node_id << ":\n";
						std::vector<std::vector<Node<3>>> buffer = aml.getNodesNeighbours();
						printNodes<3>(buffer[node_id].begin(), buffer[node_id].end());
					}

					break;
				}
				
				case 7:
				{
					aml.insertInCentres();
					std::cout << "\nNew nodes were inserted at edges centres.\n\n";

					break;
				}

				case 8:
				{
					std::cout << "All nodes:\n\n";
					printNodes<3>(aml.getNodeContainer().begin(), aml.getNodeContainer().end());

					break;
				}

				case 9:
				{
					uint32_t left_id, right_id; //nodes ID(diapazon boundaries)
					std::cout << "\nNodes ID are in diapazon [1, " << aml.getNodeCnt() << "]\n\n";
					std::cout << "Input node ID(left diapazon boundary): "; std::cin >> left_id;
					std::cout << "Input node ID(right diapazon boundary): "; std::cin >> right_id;
					if (left_id > right_id) std::cout << "\n\nEmpty diapazon!\n";
					else if ((left_id <= 0) || (right_id > aml.getNodeCnt())) std::cout << "\n\nThere are wrong values in given diapazon!\n";
					else
					{
						std::cout << "\n\nNodes with id in diapazon [" << left_id << ", " << right_id << "]:\n";
						printNodes<3>(aml.getNodeContainer().begin() + left_id - 1, aml.getNodeContainer().begin() + right_id);
					}

					break;
				}

				case 10:
				{
					std::cout << "All finite elements:\n\n";
					printFEs(aml.getFEContainer().begin(), aml.getFEContainer().end());

					break;
				}

				case 11:
				{
					uint32_t left_id, right_id; //FE ID(diapazon boundaries)
					std::cout << "\nFinite elements ID are in diapazon [1, " << aml.getFECnt() << "]\n\n";
					std::cout << "Input FE ID(left diapazon boundary): "; std::cin >> left_id;
					std::cout << "Input FE ID(right diapazon boundary): "; std::cin >> right_id;
					if (left_id > right_id) std::cout << "\n\nEmpty diapazon!\n";
					else if ((left_id <= 0) || (right_id > aml.getFECnt())) std::cout << "\n\nThere are wrong values in given diapazon!\n";
					else
					{
						std::cout << "\n\nFinite elements with id in diapazon [" << left_id << ", " << right_id << "]:\n";
						printFEs(aml.getFEContainer().begin() + left_id - 1, aml.getFEContainer().begin() + right_id);
					}

					break;
				}

				case 12:
				{
					std::vector<BoundaryFiniteElement> buffer;
					std::multimap<uint32_t, BoundaryFiniteElement> all_bfe = aml.getBFEContainer();
					
					
					for (auto cur_bfe_it = all_bfe.begin(); cur_bfe_it != all_bfe.end(); ++cur_bfe_it)
					{
						buffer.push_back(cur_bfe_it->second);
					}
					
					std::cout << "All boundary finite elements:\n\n";
					
					printBFEs(buffer.begin(), buffer.end());

					break;
				}
				
				case 13:
				{
					uint32_t b_id; //boundary ID
					std::cout << "\nInput boundary ID: "; std::cin >> b_id;
					std::cout << "\n\nBoundary finite elements with boundary id " << b_id << ":\n";
					aml.printBoundaryBFEs(b_id);

					break;
				}

				case 14:
					std::cout << "\n\nBye!\n\n";

					break;

				default:
					std::cout << "\n\nIncorrect input! Try again.\n\n";
					
				}
			}
			catch (const std::exception& exc)
			{
				std::cerr << exc.what();
			}
		}
	}
	catch (FileExc file_exc)
	{
		std::cerr << "\n" << file_exc.what() << "\n";
	}
	
	return 0;
}
